package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletGPSActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Coordinates")
    public @ActionOutput(name = "latitude", type="long")
           @ActionOutput(name = "ns", type="char")
           @ActionOutput(name = "longitude", type="long")
           @ActionOutput(name = "ew", type="char")
           @ActionOutput(name = "pdop", type="int")
           @ActionOutput(name = "hdop", type="int")
           @ActionOutput(name = "vdop", type="int")
           @ActionOutput(name = "epe", type="int")
           Map<String, Object> brickletGPSGetCoordinates(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPS.Coordinates value = ((BrickletGPS)this.handler.getDevice()).getCoordinates();
        result.put("latitude", value.latitude);
        result.put("ns", value.ns);
        result.put("longitude", value.longitude);
        result.put("ew", value.ew);
        result.put("pdop", value.pdop);
        result.put("hdop", value.hdop);
        result.put("vdop", value.vdop);
        result.put("epe", value.epe);
        return result;
    }

    public static Map<String, Object> brickletGPSGetCoordinates(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            return ((BrickletGPSActions) actions).brickletGPSGetCoordinates();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }

    @RuleAction(label = "Get Status")
    public @ActionOutput(name = "fix", type="short")
           @ActionOutput(name = "satellitesView", type="short")
           @ActionOutput(name = "satellitesUsed", type="short")
           Map<String, Object> brickletGPSGetStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPS.Status value = ((BrickletGPS)this.handler.getDevice()).getStatus();
        result.put("fix", value.fix);
        result.put("satellitesView", value.satellitesView);
        result.put("satellitesUsed", value.satellitesUsed);
        return result;
    }

    public static Map<String, Object> brickletGPSGetStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            return ((BrickletGPSActions) actions).brickletGPSGetStatus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }

    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type="int")
           @ActionOutput(name = "geoidalSeparation", type="int")
           Map<String, Object> brickletGPSGetAltitude(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPS.Altitude value = ((BrickletGPS)this.handler.getDevice()).getAltitude();
        result.put("altitude", value.altitude);
        result.put("geoidalSeparation", value.geoidalSeparation);
        return result;
    }

    public static Map<String, Object> brickletGPSGetAltitude(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            return ((BrickletGPSActions) actions).brickletGPSGetAltitude();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }

    @RuleAction(label = "Get Motion")
    public @ActionOutput(name = "course", type="long")
           @ActionOutput(name = "speed", type="long")
           Map<String, Object> brickletGPSGetMotion(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPS.Motion value = ((BrickletGPS)this.handler.getDevice()).getMotion();
        result.put("course", value.course);
        result.put("speed", value.speed);
        return result;
    }

    public static Map<String, Object> brickletGPSGetMotion(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            return ((BrickletGPSActions) actions).brickletGPSGetMotion();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }

    @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "date", type="long")
           @ActionOutput(name = "time", type="long")
           Map<String, Object> brickletGPSGetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPS.DateTime value = ((BrickletGPS)this.handler.getDevice()).getDateTime();
        result.put("date", value.date);
        result.put("time", value.time);
        return result;
    }

    public static Map<String, Object> brickletGPSGetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            return ((BrickletGPSActions) actions).brickletGPSGetDateTime();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }

    @RuleAction(label = "Restart")
    public void brickletGPSRestart(
            @ActionInput(name = "restartType") short restartType) throws TinkerforgeException {
        ((BrickletGPS)this.handler.getDevice()).restart(restartType);
    }

    public static void brickletGPSRestart(@Nullable ThingActions actions, short restartType) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            ((BrickletGPSActions) actions).brickletGPSRestart(restartType);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletGPSGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPS.Identity value = ((BrickletGPS)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletGPSGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSActions) {
            return ((BrickletGPSActions) actions).brickletGPSGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSActions class.");
        }
    }
}
