package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOneWireActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Search Bus")
    public @ActionOutput(name = "identifier", type="long[]")
           @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireSearchBus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOneWire.SearchBus value = ((BrickletOneWire)this.handler.getDevice()).searchBus();
        result.put("identifier", value.identifier);
        result.put("status", value.status);
        return result;
    }

    public static Map<String, Object> brickletOneWireSearchBus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireSearchBus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Reset Bus")
    public @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireResetBus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).resetBus();
        result.put("status", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireResetBus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireResetBus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Write")
    public @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireWrite(
            @ActionInput(name = "data") int data) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).write(data);
        result.put("status", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireWrite(@Nullable ThingActions actions, int data) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireWrite(data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Read")
    public @ActionOutput(name = "data", type="int")
           @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireRead(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOneWire.Read value = ((BrickletOneWire)this.handler.getDevice()).read();
        result.put("data", value.data);
        result.put("status", value.status);
        return result;
    }

    public static Map<String, Object> brickletOneWireRead(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireRead();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Write Command")
    public @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireWriteCommand(
            @ActionInput(name = "identifier") long identifier,
            @ActionInput(name = "command") int command) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).writeCommand(identifier, command);
        result.put("status", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireWriteCommand(@Nullable ThingActions actions, long identifier, int command) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireWriteCommand(identifier, command);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Get Communication LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletOneWireGetCommunicationLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).getCommunicationLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetCommunicationLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireGetCommunicationLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletOneWireReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletOneWire)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletOneWireGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletOneWireGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletOneWireGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOneWire)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletOneWireGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOneWire.SPITFPErrorCount value = ((BrickletOneWire)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOneWireGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOneWire.Identity value = ((BrickletOneWire)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOneWireActions) {
            return ((BrickletOneWireActions) actions).brickletOneWireGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOneWireActions class.");
        }
    }
}
