/* ***********************************************************
 * This file was automatically generated on 2020-01-23.      *
 *                                                           *
 * OpenHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.Arrays;
import java.util.List;
import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.eclipse.smarthome.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.DecimalType;

/**
 * Creates beep with configurable frequency
 */
public class BrickletPiezoSpeaker extends Device {
	public final static int DEVICE_IDENTIFIER = 242;
	public final static String DEVICE_DISPLAY_NAME = "Piezo Speaker Bricklet";

    public final static DeviceInfo DEVICE_INFO = new DeviceInfo(DEVICE_DISPLAY_NAME, "brickletpiezospeaker", DEVICE_IDENTIFIER, BrickletPiezoSpeaker.class, BrickletPiezoSpeakerActions.class, "2.0.3");

	public final static byte FUNCTION_BEEP = (byte)1;
	public final static byte FUNCTION_MORSE_CODE = (byte)2;
	public final static byte FUNCTION_CALIBRATE = (byte)3;
	public final static byte FUNCTION_GET_IDENTITY = (byte)255;
	private final static int CALLBACK_BEEP_FINISHED = 4;
	private final static int CALLBACK_MORSE_CODE_FINISHED = 5;

	public final static long BEEP_DURATION_OFF = 0L;
	public final static long BEEP_DURATION_INFINITE = 4294967295L;

	private List<BeepFinishedListener> listenerBeepFinished = new CopyOnWriteArrayList<BeepFinishedListener>();
	private List<MorseCodeFinishedListener> listenerMorseCodeFinished = new CopyOnWriteArrayList<MorseCodeFinishedListener>();

	/**
	 * This listener is triggered if a beep set by {@link BrickletPiezoSpeaker#beep(long, int)} is finished
	 */
	public interface BeepFinishedListener extends DeviceListener {
		public void beepFinished();
	}

	/**
	 * This listener is triggered if the playback of the morse code set by
	 * {@link BrickletPiezoSpeaker#morseCode(String, int)} is finished.
	 */
	public interface MorseCodeFinishedListener extends DeviceListener {
		public void morseCodeFinished();
	}

	/**
	 * Creates an object with the unique device ID \c uid. and adds it to
	 * the IP Connection \c ipcon.
	 */
	public BrickletPiezoSpeaker(String uid, IPConnection ipcon) {
		super(uid, ipcon);

		apiVersion[0] = 2;
		apiVersion[1] = 0;
		apiVersion[2] = 0;

		deviceIdentifier = DEVICE_IDENTIFIER;
		deviceDisplayName = DEVICE_DISPLAY_NAME;

		responseExpected[IPConnection.unsignedByte(FUNCTION_BEEP)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_MORSE_CODE)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_CALIBRATE)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_GET_IDENTITY)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;

		callbacks[CALLBACK_BEEP_FINISHED] = new IPConnection.DeviceCallbackListener() {
			public void callback(byte[] packet) {
				for (BeepFinishedListener listener: listenerBeepFinished) {
					listener.beepFinished();
				}
			}
		};

		callbacks[CALLBACK_MORSE_CODE_FINISHED] = new IPConnection.DeviceCallbackListener() {
			public void callback(byte[] packet) {
				for (MorseCodeFinishedListener listener: listenerMorseCodeFinished) {
					listener.morseCodeFinished();
				}
			}
		};
	}

	/**
	 * Beeps with the given frequency for the given duration.
	 * 
	 * .. versionchanged:: 2.0.2$nbsp;(Plugin)
	 *    A duration of 0 stops the current beep if any, the frequency parameter is
	 *    ignored. A duration of 4294967295 results in an infinite beep.
	 * 
	 * The Piezo Speaker Bricklet can only approximate the frequency, it will play
	 * the best possible match by applying the calibration (see {@link BrickletPiezoSpeaker#calibrate()}).
	 */
	public void beep(long duration, int frequency) throws TinkerforgeException {
		checkDeviceIdentifier();

		ByteBuffer bb = ipcon.createRequestPacket((byte)14, FUNCTION_BEEP, this);

		bb.putInt((int)duration);
		bb.putShort((short)frequency);

		sendRequest(bb.array());
	}

	/**
	 * Sets morse code that will be played by the piezo buzzer. The morse code
	 * is given as a string consisting of "." (dot), "-" (minus) and " " (space)
	 * for *dits*, *dahs* and *pauses*. Every other character is ignored.
	 * 
	 * For example: If you set the string "...---...", the piezo buzzer will beep
	 * nine times with the durations "short short short long long long short
	 * short short".
	 */
	public void morseCode(String morse, int frequency) throws TinkerforgeException {
		checkDeviceIdentifier();

		ByteBuffer bb = ipcon.createRequestPacket((byte)70, FUNCTION_MORSE_CODE, this);

		for (int i = 0; i < 60; i++) {
			try {
				bb.put((byte)morse.charAt(i));
			} catch(Exception e) {
				bb.put((byte)0);
			}
		}
		bb.putShort((short)frequency);

		sendRequest(bb.array());
	}

	/**
	 * The Piezo Speaker Bricklet can play 512 different tones. This function
	 * plays each tone and measures the exact frequency back. The result is a
	 * mapping between setting value and frequency. This mapping is stored
	 * in the EEPROM and loaded on startup.
	 * 
	 * The Bricklet should come calibrated, you only need to call this
	 * function (once) every time you reflash the Bricklet plugin.
	 * 
	 * Returns *true* after the calibration finishes.
	 */
	public boolean calibrate() throws TinkerforgeException {
		checkDeviceIdentifier();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_CALIBRATE, this);


		byte[] response = sendRequest(bb.array());

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		boolean calibration = (bb.get()) != 0;

		return calibration;
	}

	/**
	 * Returns the UID, the UID where the Bricklet is connected to,
	 * the position, the hardware and firmware version as well as the
	 * device identifier.
	 * 
	 * The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
	 * The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
	 * connected to an :ref:`Isolator Bricklet &lt;isolator_bricklet&gt;` is always as
	 * position 'z'.
	 * 
	 * The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
	 * |device_identifier_constant|
	 */
	public Identity getIdentity() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_IDENTITY, this);


		byte[] response = sendRequest(bb.array());

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		Identity obj = new Identity();
		obj.uid = IPConnection.string(bb, 8);
		obj.connectedUid = IPConnection.string(bb, 8);
		obj.position = (char)(bb.get());
		for (int i = 0; i < 3; i++) {
			obj.hardwareVersion[i] = IPConnection.unsignedByte(bb.get());
		}
		for (int i = 0; i < 3; i++) {
			obj.firmwareVersion[i] = IPConnection.unsignedByte(bb.get());
		}
		obj.deviceIdentifier = IPConnection.unsignedShort(bb.getShort());

		return obj;
	}

	/**
	 * Adds a BeepFinished listener.
	 */
	public void addBeepFinishedListener(BeepFinishedListener listener) {
		listenerBeepFinished.add(listener);
	}

	/**
	 * Removes a BeepFinished listener.
	 */
	public void removeBeepFinishedListener(BeepFinishedListener listener) {
		listenerBeepFinished.remove(listener);
	}

	/**
	 * Adds a MorseCodeFinished listener.
	 */
	public void addMorseCodeFinishedListener(MorseCodeFinishedListener listener) {
		listenerMorseCodeFinished.add(listener);
	}

	/**
	 * Removes a MorseCodeFinished listener.
	 */
	public void removeMorseCodeFinishedListener(MorseCodeFinishedListener listener) {
		listenerMorseCodeFinished.remove(listener);
	}

    private final Logger logger = LoggerFactory.getLogger(BrickletPiezoSpeaker.class);
    private final static Logger static_logger = LoggerFactory.getLogger(BrickletPiezoSpeaker.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletPiezoSpeakerConfig cfg = (BrickletPiezoSpeakerConfig) config.as(BrickletPiezoSpeakerConfig.class);
        this.addBeepFinishedListener(() -> {if(true) {triggerChannelFn.accept("BrickletPiezoSpeakerBeepFinished", transformBrickletPiezoSpeakerBeepFinishedCallback0(cfg));}});
		this.addMorseCodeFinishedListener(() -> {if(true) {triggerChannelFn.accept("BrickletPiezoSpeakerMorseCodeFinished", transformBrickletPiezoSpeakerMorseCodeFinishedCallback0(cfg));}});
        
		{
		
		}
		{
		
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        BrickletPiezoSpeakerConfig cfg = (BrickletPiezoSpeakerConfig) config.as(BrickletPiezoSpeakerConfig.class);
        this.listenerBeepFinished.clear();
		this.listenerMorseCodeFinished.clear();
        
		
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        BrickletPiezoSpeakerConfig cfg = (BrickletPiezoSpeakerConfig) config.as(BrickletPiezoSpeakerConfig.class);
        List<String> result = new ArrayList<String>();
        result.add("BrickletPiezoSpeakerBeepFinished");
		result.add("BrickletPiezoSpeakerMorseCodeFinished");
        return result;
    }

    public static ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            
            default:
                static_logger.debug("Unknown channel type ID {}", channelTypeUID.getId());
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimum_fw_version);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Piezo Speaker Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Creates beep with configurable frequency").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletPiezoSpeakerBeepFinished", new ChannelTypeUID("system", "trigger")).withLabel("Beep Finished").withDescription("This channel is triggered if a beep set by the beep action is finished.").build(), new ChannelDefinitionBuilder("BrickletPiezoSpeakerMorseCodeFinished", new ChannelTypeUID("system", "trigger")).withLabel("Morse Code Finished").withDescription("This channel is triggered if the playback of the morse code set by the morseCode action is finished.").build())).withProperties(props).build();
    }

    public static ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletpiezospeaker":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            default:
                static_logger.debug("Unknown config description URI {}", uri.toASCIIString());
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletPiezoSpeakerConfig cfg = (BrickletPiezoSpeakerConfig) config.as(BrickletPiezoSpeakerConfig.class);
        switch(channel) {
            case "BrickletPiezoSpeakerBeepFinished":
               return;
            case "BrickletPiezoSpeakerMorseCodeFinished":
               return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        BrickletPiezoSpeakerConfig cfg = (BrickletPiezoSpeakerConfig) config.as(BrickletPiezoSpeakerConfig.class);
        switch(channel) {
            
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private String transformBrickletPiezoSpeakerBeepFinishedCallback0(BrickletPiezoSpeakerConfig cfg) {
        return "";
    }
	    private String transformBrickletPiezoSpeakerMorseCodeFinishedCallback0(BrickletPiezoSpeakerConfig cfg) {
        return "";
    }
}
